
%   This program loads data on interest rates and estimates a discrete diffusion
%   having one lagged value of the interest rate as the single explanatory variable.
%   The (general) continuous time diffusion is,
%   dr(t) = [a+br(t)]dt+sigma[r(t)^c]dW(t)
%
%   The exact discrete version is,
%   r(t) = exp(b)*r(t-1)+(a/b)(e^b-1)+u(t)
%   with u(t) Normal, zero mean, and conditional variance,
%   nu(t)^2 =[sigma^2/(2b)][e^(2b)-1][r(t-1)^(2*c)]
%
%   It starts by computing initial estimates through a least-squares regresion [AR(1)]
%   It then estimates a regression of the squared residuals on the logged value of the first 
%   lag of interest rates to obtain initial estimates for the variance parameters.
%   Alfonso Novales

clear;
load tipos.prn -ascii;

r=tipos(:,1);
T=size(r,1);

lagr=r(1:T-1,1);
r=r(2:T,1);

%  To estimate initial conditions for the exact discretization, we estimate an AR(1) in 
%  interest rates and solve the nonlinear relationships with the coefficients in the 
%  discrete approximation

[bhat,ehat,sigma2,sigbhat,estadt,pvalor,R2]=mco_breve(r,[ones(T-1,1) lagr]);
bet = log(bhat(2));
alp = bhat(1)*bet/(bhat(2)-1);
gam = 0.5;    %  The CIR restriction

eps = log(ehat.^2);
[bhat,ehat2,sigma2,sigbhat,estadt,pvalor,R2]=mco_breve(eps,[ones(T-1,1) log(lagr)]);

%  The sigma parameter in the exact and the CKLS approximations are not the same.
%  We compute the one for the exact app. from the estimate for the CKLS
%  approx.

sig = (exp(bhat(1))*2*bet/(exp(2*bet)-1))^gam;

parm = [alp;bet;sig;gam];
[pcnew,iter,fnew,gnew,hessnew] = eemin5('CIR',parm,'',r,lagr)

[result,vart2] = CIR(pcnew,r,lagr);
varianza = mean(vart2);
dt=sqrt(diag(inv(hessnew)));
%  dt=sqrt(varianza).*dt;

display('maximized likelihood')
target = -result
display('estimates    std    t-ratios')
[pcnew dt pcnew./dt]
rhat = exp(pcnew(2))*lagr+pcnew(1)/pcnew(2)*(exp(pcnew(2))-1);
scatter(r,rhat);